/*
 * playlistaspectlistviewrenderer.h
 *
 *  Created on: 24.04.2010
 *      Author: darkstar
 */

#ifndef PLAYLISTASPECTLISTVIEWRENDERER_H_
#define PLAYLISTASPECTLISTVIEWRENDERER_H_

#include "playlistaspect.h"
#include "media.h"

class PlayListAspectItem: public MediaListener
{
public:
	PlayListAspectItem(unsigned long mediaID, unsigned long playlistID, MediaDatabase *mediaDatabase)
		: mediaID_(mediaID), id_(playlistID), media_(NULL), mediaDatabase_(mediaDatabase) {}

	virtual ~PlayListAspectItem() { if (media_) media_->unregisterListener(this); }

	Media *media() const { return media_ ? media_ : getAndSetMedia(); }

	inline unsigned long id() { return id_; }
	inline void setID(unsigned long newID) { id_ = newID; }
	inline unsigned long mediaID() { return mediaID_; }
	inline void setMediaID(unsigned long newID) { mediaID_ = newID; }

protected:
	void mediaDestroyed(Media *) { media_ = NULL; }
	Media *getAndSetMedia() const;

private:
	friend class PlayListAspectListViewRenderer;
	unsigned long mediaID_;
	unsigned long id_;
	Media *media_;
	MediaDatabase *mediaDatabase_;
};

#ifdef QT4
typedef QList<PlayListAspectItem *> PlayListAspectItemList;
#else
typedef QList<PlayListAspectItem> PlayListAspectItemList;
#endif

class PlayListAspectListViewRenderer: public QObject, public PlayListAspectExtension
{
	Q_OBJECT
public:
	enum Columns {
		COLUMN_NONE = 0,
		COLUMN_TRACKNO = 1,
		COLUMN_TITLE = 2,
		COLUMN_ARTIST = 3,
		COLUMN_ALBUM = 4,
		COLUMN_LENGTH = 5,
		COLUMN_MAX = 6
	};

	PlayListAspectListViewRenderer();
	virtual ~PlayListAspectListViewRenderer();

	virtual void execute();
	virtual void reset();
	virtual const QString outputViewSource();

	virtual bool isActive() { return true; }
	virtual bool isFiltering() { return isSorted_; }

	virtual void saveExtensionState(QStringList &dst);
	virtual void loadExtensionState(QStringList &src);

	int sortedColumn() { return sortedColumn_; }
	bool isSortAscending() { return isSortAscending_; }
	void setSortParams(int column, bool ascending = TRUE);
	void resetSorting();
	bool isSorted() { return isSorted_; };

	int selectedItemIndex() { return selectedItemIndex_; }
	void setSelectedItemIndex(int index) { selectedItemIndex_ = index; }

	QPoint viewportPosition() { return viewportPosition_; }
	void setViewportPosition(QPoint pos) { viewportPosition_ = pos; }

	const PlayListAspectItemList &itemList() const;

signals:
	void updateView();

private:
 	bool isSorted_;
 	bool isSortAscending_;
 	int sortedColumn_;
	int selectedItemIndex_;
	QPoint viewportPosition_;
 	bool itemListDirty_;

	PlayListAspectItemList items_;

 	void updateItemList();
 	void clearItemList();
};

#endif /* PLAYLISTASPECTLISTVIEWRENDERER_H_ */
